@@ -23,6 +23,7 @@ def main():
     from .compiler import compile
     from .source import filters
     from .source.colors import Color
+    from .source.parsers import parsers, AbstractParser
 
     parser = argparse.ArgumentParser(
         "bake",
@@ -157,18 +158,22 @@ def main():
             return Color(r, g, b)
 
     parser.add_argument("reports", type=PathSorter(reports, "report"), default=None, nargs="*" if "BAGUETTE_REPORTS" in environ else "+", help="Cuckoo report files (.json) to bake baguettes from. Can also be folders containing reports. Defaults to environment variable 'BAGUETTE_REPORTS' if set.")
-    parser.add_argument("--baguettes", type=PathSorter(baguettes, "baguette"), default=None, action="extend", nargs="*", help="The path(s) to the output (copy) baguette files (.pyt). Use '-' to leave it to the automatic destination. Defaults to environment variable 'BAGUETTE_BAGUETTES' or '-' if not set.")
-    parser.add_argument("--visuals", type=PathSorter(visuals, "visual"), default=None, action="extend", nargs="*", help="The path(s) to the output (copy) visual (Gephi) files (.gexf). Use '-' to leave it to the automatic destination. Defaults to environment variable 'BAGUETTE_VISUALS' or '-' if not set.")
-    parser.add_argument("-o", "--outputs", type=PathSorter(outputs, "output"), default=None, action="extend", nargs="*", help="The path to the result index folders (which end in .bag). They contain the index file (.pyt) which stores all the information about a given baguette. Use '-' to leave it to the automatic destination. Defaults to environment variable 'BAGUETTE_OUTPUTS' or '.' if not set.")
-    parser.add_argument("--pool", type=pool_size, default=pool_size("0.5"), help="The size of the process pool to use to bake in parallel.")
-    parser.add_argument("--maxtime", type=time, default=time("inf"), help="The maximum amount of time spent baking a single baguette. No maxtime by default.")
-    parser.add_argument("-f", "--filters", type=str, default=[], choices=[name for name in dir(filters) if isinstance(getattr(filters, name), filters.Filter)], nargs="*", help="A list of filters that can be used when exporting the baguette to the visual file (.gexf).")
+    
+    parser.add_argument("--baguettes", "-B", type=PathSorter(baguettes, "baguette"), default=None, action="extend", nargs="*", help="The path(s) to the output (copy) baguette files (.pyt). Use '-' to leave it to the automatic destination. Defaults to environment variable 'BAGUETTE_BAGUETTES' or '-' if not set.")
+    parser.add_argument("--visuals", "-V", type=PathSorter(visuals, "visual"), default=None, action="extend", nargs="*", help="The path(s) to the output (copy) visual (Gephi) files (.gexf). Use '-' to leave it to the automatic destination. Defaults to environment variable 'BAGUETTE_VISUALS' or '-' if not set.")
+    parser.add_argument("--outputs", "-o", type=PathSorter(outputs, "output"), default=None, action="extend", nargs="*", help="The path to the result index folders (which end in .bag). They contain the index file (.pyt) which stores all the information about a given baguette. Use '-' to leave it to the automatic destination. Defaults to environment variable 'BAGUETTE_OUTPUTS' or '.' if not set.")
+    parser.add_argument("--pool", "-p", type=pool_size, default=pool_size("0.5"), help="The size of the process pool to use to bake in parallel.")
+    parser.add_argument("--maxtime", "-m", type=time, default=time("inf"), help="The maximum amount of time spent baking a single baguette. No maxtime by default.")
+    parser.add_argument("--report_type", "-t", type=str, default=AbstractParser.report_name, choices=[p.report_name for p in parsers], help="The type of execution report used as a source. Tries to autodetect for each sample by default.")
+    parser.add_argument("--filters", "-F", type=str, default=[], choices=[name for name in dir(filters) if isinstance(getattr(filters, name), filters.Filter)], nargs="*", help="A list of filters that can be used when exporting the baguette to the visual file (.gexf).")
+    parser.add_argument("--idempotent", "-i", action="store_true", default=False, help="If enabled, the compiler will first search for a compiled BAGUETTE graph in the computed output folder. It will (re)compile it if it does not exist, if it timed out or if had different compilation parameters.")
+    parser.add_argument("--save_filtered_baguette", "-sv", action="store_true", default=False, help="If enabled, instead of saving the raw BAGUETTE graph in the 'baguette.pyt' file, only the filtered BAGUETTE will be saved.")
+    parser.add_argument("--background", "-b", type=paint_color, default=Color.black, help="If a color is given for background, the color settings which are close to the background color will be changed to be more visible on that background. Must be a valid color name or RGB values.")
+    parser.add_argument("--verbosity", "-v", action="count", default=0, help="Increases the verbosity of the output.")
+    
     parser.add_argument("--perf", action="store_true", default=False, help="If this is enabled, a performance report will be printed at the end of the baking process.")
-    parser.add_argument("-v", "--verbosity", action="count", default=0, help="Increases the verbosity of the output.")
     parser.add_argument("--skip_data_comparison", action="store_true", default=False, help="If enabled, the computation of the Levenshtein similarity between all Data nodes will be skipped.")
     parser.add_argument("--skip_diff_comparison", action="store_true", default=False, help="Same as skip_data_comparison but for Diff nodes.")
-    parser.add_argument("--save_filtered_baguette", action="store_true", default=False, help="If enabled, instead of saving the raw BAGUETTE graph in the 'baguette.pyt' file, only the filtered BAGUETTE will be saved.")
-    parser.add_argument("--background", type=paint_color, default=Color.black, help="If a color is given for background, the color settings which are close to the background color will be changed to be more visible on that background. Must be a valid color name or RGB values.")
 
 
     args = parser.parse_args()
@@ -311,7 +316,7 @@ def main():
         """
         Given a path, this will return a (possibly) modified path that does not exist in the same folder.
         """
-        if not folder.exists():
+        if args.idempotent or not folder.exists():
             return folder
         n = 0
         ext = folder.suffix
@@ -451,6 +456,7 @@ def main():
             bg.perf = args.perf
             bg.background_color = args.background
             bg.save_filtered_baguette = args.save_filtered_baguette
+            bg.report_type = args.report_type
             work.append((bg, r, b, v))
     
     
@@ -470,9 +476,26 @@ def main():
                 return False
             br, r, b, v = work.pop()
         try:
-            br.export()
-            P.apply(compile, (br.index, r, b, v))
-            br = BaguetteRack.import_from(br.index)
+            skip = False
+            if args.idempotent and br.exists():
+                old_br = BaguetteRack.import_from(br.index)
+                if (
+                    old_br.compilation_interrupted
+                    or (not old_br.baked and not old_br.exception)
+                    or br.background_color != old_br.background_color
+                    or br.filter_names != old_br.filter_names
+                    or br.save_filtered_baguette != old_br.save_filtered_baguette
+                    or br.skip_data_comparison != old_br.skip_data_comparison
+                    or br.skip_diff_comparison != old_br.skip_diff_comparison
+                    ):
+                    old_br.clean()
+                else:
+                    skip = True
+                    logging.info(f"Skipping '{r}': Already baked")
+            if not skip:
+                br.export()
+                P.apply(compile, (br.index, r, b, v))
+                br = BaguetteRack.import_from(br.index)
         except KeyboardInterrupt as e:
             from traceback import TracebackException
             br.exception = TracebackException.from_exception(e)
