@@ -93,7 +93,7 @@ def compile(rack_path : Path, report_path : Path, cp_baguette : Path | None = No
                 b.build()
                 G = b.graph
                 logger.info("Analyzing graph...")
-                logger.info("Got {} vertices and {} edges.".format(G.n, G.m))
+                logger.info("Got {} vertices and {} edges.".format(len(G.vertices), len(G.edges)))
                 if not rack.save_filtered_baguette:
                     logger.info("Saving with pickle")
                     rack.baguette.parent.mkdir(parents = True, exist_ok = True)
