@@ -4,9 +4,8 @@ Look at the Event class.
 """
 
 from typing import Callable, Generic, Type, TypeVar
-from Viper.better_threading import Future
+from Boa.parallel.thread import Future
 from threading import Lock
-from .utils import chrono
 
 __all__ = ["Event"]
 
@@ -46,7 +45,6 @@ class Event(Generic[T]):
         """
         return type(self).__name__ + "(" + ", ".join(str(name) + " = " + repr(getattr(self, name)) for name in self.__slots__) + ")"
 
-    @chrono
     def throw(self : T) -> bool:
         """
         Triggers the Event. Everyone waiting for it will be awaken, and corresponding callbacks will be called.
@@ -106,7 +104,7 @@ class Event(Generic[T]):
         """
         Waits for an Event of this class or of any subclasses to occur. Returns the Event object when it happens.
         """
-        from Viper.better_threading import Future
+        from Boa.parallel.thread import Future
         fut : Future[T] = Future()
         with Event.__locks[cls]:
             Event.__waiting[cls].append(fut)
