@@ -9,6 +9,7 @@ from Viper.meta.iterable import InstanceReferencingClass
 from .utils import chrono, System
 from .types.registry import Key, KeyEntry, Handle
 from functools import cache
+from Viper.collections.isomorph import IsoSet
 
 __all__ = ["Filter", "significant_call_only", "no_data_nodes", "no_simple_imports", "no_handle_nodes", "injected_threads_only", "significant_processes_only", "modified_registry_only"]
 
@@ -68,10 +69,11 @@ class Filter(metaclass = InstanceReferencingClass):
         Applies the filter to the given graph, modifying it.
         """
         from .graph import Graph
+        from Viper.collections.isomorph import IsoSet
         if not isinstance(g, Graph):
             raise TypeError("Expected Graph, got " + repr(type(g).__name__))
-        edges_to_remove = {e for e in g.edges if not self.__edge_filter(e)}
-        vertices_to_remove = {u for u in g.vertices if not self.__vertex_filter(u)}
+        edges_to_remove = IsoSet(e for e in g.edges if not self.__edge_filter(e))
+        vertices_to_remove = IsoSet(u for u in g.vertices if not self.__vertex_filter(u))
         for e in edges_to_remove:
             g.remove(e)
         for u in vertices_to_remove:
@@ -136,7 +138,7 @@ def single_processes(u : Vertex) -> bool:
 
 
 Key_KeyEntry_Model : System[Key | KeyEntry | Handle] = System()
-_strong_keys : set[Key] = set()
+_strong_keys : IsoSet[Key] = IsoSet()
 
 @cache
 def changed_registry(u : Vertex) -> bool:           # This filter is so fucking complicated!!!
@@ -212,4 +214,4 @@ injected_threads_only = Filter(lambda e : True, single_threads)
 significant_processes_only = Filter(lambda e : True, single_processes)
 modified_registry_only = Filter(lambda e : True, changed_registry)
 
-del important_call_filter, data_filter, simple_imports, no_handles, single_threads, Callable, Iterable, Edge, Vertex, Graph, InstanceReferencingClass, Key, KeyEntry
\ No newline at end of file
+del important_call_filter, data_filter, simple_imports, no_handles, single_threads, Callable, Iterable, Edge, Vertex, Graph, InstanceReferencingClass, Key, KeyEntry, IsoSet
\ No newline at end of file
