@@ -22,23 +22,11 @@ def register_read_operation(target : File | Connection, vector : Handle | Socket
     """
     Registers a read operation to be added to the corresponding DiffNodes.
     """
-    from .entities import Diff
-    from .relations import IsDiffOf
-    from .utils import Read
+    from .utils import Read, PreDiffDescriptor
     if isinstance(content, str):
         content = content.encode("utf-8")
-    if target not in Diff._active_target_diff:
-        t = Diff()
-        IsDiffOf(t, target)
-        Diff._active_target_diff[target] = t
-    else:
-        t = Diff._active_target_diff[target]
-    if vector not in Diff._active_vector_diff:
-        v = Diff()
-        IsDiffOf(v, vector)
-        Diff._active_vector_diff[vector] = v
-    else:
-        v = Diff._active_vector_diff[vector]
+    t = PreDiffDescriptor.get_target_diff_descriptor(target)
+    v = PreDiffDescriptor.get_vector_diff_descriptor(vector)
     
     if offset == None:
         t.add_operation(Read(t.last_pos(), content))
@@ -52,23 +40,11 @@ def register_write_operation(target : File | Connection, vector : Handle | Socke
     """
     Registers a write operation to be added to the corresponding DiffNodes.
     """
-    from .entities import Diff
-    from .relations import IsDiffOf
-    from .utils import Write
+    from .utils import Write, PreDiffDescriptor
     if isinstance(content, str):
         content = content.encode("utf-8")
-    if target not in Diff._active_target_diff:
-        t = Diff()
-        IsDiffOf(t, target)
-        Diff._active_target_diff[target] = t
-    else:
-        t = Diff._active_target_diff[target]
-    if vector not in Diff._active_vector_diff:
-        v = Diff()
-        IsDiffOf(v, vector)
-        Diff._active_vector_diff[vector] = v
-    else:
-        v = Diff._active_vector_diff[vector]
+    t = PreDiffDescriptor.get_target_diff_descriptor(target)
+    v = PreDiffDescriptor.get_vector_diff_descriptor(vector)
     
     if offset == None:
         t.add_operation(Write(t.last_pos(), content))
@@ -77,13 +53,13 @@ def register_write_operation(target : File | Connection, vector : Handle | Socke
         t.add_operation(Write(offset, content))
         v.add_operation(Write(offset, content))
 
+
 __N_diff_comparison_phase = BuildingPhase.request_finalizing_phase()
 __N_diff_normalization_phase = BuildingPhase.request_finalizing_phase()
-__N_diff_fusion_phase = BuildingPhase.request_finalizing_phase()
         
 
 @chrono
-def finalize_diff_nodes(e : BuildingPhase):
+def compile_diff_nodes(e : BuildingPhase):
     """
     When called with the right finalizing phase event, will cause all Diff nodes to compute their data attributes.
     """
@@ -94,21 +70,27 @@ def finalize_diff_nodes(e : BuildingPhase):
     from .....logger import logger
     from ...config import CompilationParameters
     from .entities import Diff
+    from .utils import PreDiffDescriptor
     if e.major == "Finalizer" and e.minor == __N_diff_comparison_phase:
-        logger.debug("Finalizing{} {} Diff nodes.".format(" and comparing" if not CompilationParameters.SkipLevenshteinForDiffNodes else "",len(Diff)))
-        n = len(Diff)
+        descriptors = PreDiffDescriptor.descriptors()
+        logger.debug("Compiling{} {} Diff nodes.".format(" and comparing" if not CompilationParameters.SkipLevenshteinForDiffNodes else "", len(descriptors)))
+        n = len(descriptors)
         t0 = time_ns()
         t = t0
-        for i, d in enumerate(Diff):
-            d.compute_data()
+        for i, d in enumerate(descriptors):
+            if n == 0:
+                breakpoint()
+            Diff.process_data(d)
             if (time_ns() - t) / 1000000000 > 15:
                 t = time_ns()
-                logger.debug("Finalizing {} Diff nodes : {:.2f}%. ETA : {}".format(len(Diff), (i + 1) / n * 100, duration(round((t - t0) / (i + 1) * (n - i - 1)))))
+                logger.debug("Finalizing {} Diff nodes : {:.2f}%. ETA : {}".format(n, (i + 1) / n * 100, duration(round((t - t0) / (i + 1) * (n - i - 1)))))
 
 @chrono
 def normalize_diff_nodes(e : BuildingPhase):
     """
     When called with the right finalizing phase event, will cause all Diff nodes to compare their diff file sizes and change their size accordingly.
+    Same for colors regarding entropy using the defined color scale.
+    Also, Diff node size will also be proportional to the square root of the number of vectors and targets linked to them.
     """
     from typing import Type
 
@@ -118,6 +100,7 @@ def normalize_diff_nodes(e : BuildingPhase):
     from ..filesystem import File, Handle
     from ..network import Connection, Socket
     from .entities import Diff
+    from .relations import IsDiffOf
 
     def project_range(x : float, sa : float, sb : float, da : float, db : float) -> float:
         if sa == sb:
@@ -142,7 +125,7 @@ def normalize_diff_nodes(e : BuildingPhase):
         minsize = 0.5
         maxsize = 3.0
         for d in Diff:
-            d.size = project_range(len(d.glob), a, b, minsize, maxsize)
+            d.size = project_range(len(d.glob), a, b, minsize, maxsize) * len([e for e in d.edges if isinstance(e, IsDiffOf)]) ** 0.5
     
         for cls in (File, Handle, Socket, Connection):
             normalize_neighbor(cls)
@@ -152,56 +135,15 @@ def normalize_diff_nodes(e : BuildingPhase):
         for d in Diff:
             d.color = Color.linear((Diff.diff_low_entropy_color, Diff.diff_high_entropy_color), (1 - d.glob_entropy / 8, d.glob_entropy / 8))
 
-@chrono
-def fuse_diff_nodes(e : BuildingPhase):
-    """
-    When called with the right finalizing phase event, will cause all identical Diff nodes to be merged with their size increasing.
-    """
-    from .....logger import logger
-    from ...graph import Graph
-    from .entities import Diff
 
-    groups : dict[Diff, int] = {}
 
-    def merge(u : Diff, v : Diff):
-        """
-        Merges vertex v into vertex u (v gets deleted at the end).
-        """
-        for e in v.edges:
-            if e.source is v:
-                e.source = u
-            if e.destination is v:
-                e.destination = u
-        v.edges.clear()
-        for g in Graph.active_graphs():
-            g.remove(v)
-        groups[u] += 1
-
-    if e.major == "Finalizer" and e.minor == __N_diff_fusion_phase:
-        logger.debug("Fusing {} Diff nodes.".format(len(Diff)))
-        for u in Diff:
-            merged = False
-            for v in filter(lambda v : v in groups, Diff):
-                if u.content == v.content:
-                    merge(v, u)
-                    merged = True
-                    break
-            if not merged:
-                groups[u] = 1
-        
-        resize_work = [u for u, n in groups.items() if n > 1]
-        if resize_work:
-            logger.debug("Resizing {} fused Diff nodes. Actually lost {} Diff nodes.".format(len(resize_work), len(Diff) - len(groups)))
-            for u in resize_work:
-                u.size *= groups[u] ** 0.5
 
         
-BuildingPhase.add_callback(finalize_diff_nodes)
+BuildingPhase.add_callback(compile_diff_nodes)
 BuildingPhase.add_callback(normalize_diff_nodes)
-BuildingPhase.add_callback(fuse_diff_nodes)
 
 
 
 
 
-del BuildingPhase, Connection, File, Handle, Socket, chrono, entities, finalize_diff_nodes, fuse_diff_nodes, logger, normalize_diff_nodes, relations
\ No newline at end of file
+del BuildingPhase, Connection, File, Handle, Socket, chrono, entities, compile_diff_nodes, logger, normalize_diff_nodes, relations
\ No newline at end of file
