@@ -10,7 +10,7 @@ from Viper.meta.decorators import staticproperty
 from .....logger import logger
 from ...config import ColorSetting, SizeSetting
 from ...colors import Color
-from ...graph import UniqueVertex, Vertex
+from ...graph import DataVertex
 
 __all__ = ["Connection", "Socket", "Host"]
 
@@ -20,36 +20,81 @@ __all__ = ["Connection", "Socket", "Host"]
 
 logger.info("Loading entities from {} library.".format(__name__.rpartition(".")[0].rpartition(".")[2]))
 
-class Connection(UniqueVertex):
+InterNetworkAddress = tuple[str, int]
+
+class Connection(DataVertex):
 
     """
     A connection vertex. This is a link vertex between two socket. It might only exist for a part of the lifetime of both its connected sockets.
     """
 
     __slots__ = {
-        "duration" : "The duration that the connection was maintained for",
-        "volume" : "The amount of data transfered through the connection",
-        "src" : "The source address",
-        "dest" : "The destination address"
+        "__volume" : "The amount of data transfered through the connection",
+        "__src" : "The source address",
+        "__dst" : "The destination address"
     }
 
-    __pickle_slots__ = {
-        "duration",
+    __additional_data__ = DataVertex.__additional_data__ | {
         "volume",
         "src",
-        "dest"
+        "dst"
     }
 
     default_color = ColorSetting(Color(100, 50, 255))
     default_size = SizeSetting(1.5)
 
-    def __init__(self, *, parent: Optional[Vertex] = None) -> None:
-        from typing import Any
-        super().__init__(parent=parent)
-        self.duration : float = 0.
-        self.volume : int = 0
-        self.src : Any = None
-        self.dest : Any = None
+    def __init__(self) -> None:
+        self.__volume = 0
+        self.__src = None
+        self.__dst = None
+        super().__init__()
+
+    @property
+    def volume(self) -> int:
+        """
+        The amount of data (in bytes) exchanged during the connection.
+        """
+        return self.__volume
+    
+    @volume.setter
+    def volume(self, v : int):
+        if not isinstance(v, int):
+            raise TypeError(f"Expected int, got '{type(v).__name__}'")
+        self.__volume = v
+
+    @property
+    def src(self) -> InterNetworkAddress | None:
+        """
+        The address of the source of the connection as a tuple (ip, port).
+        """
+        return self.__src
+    
+    @src.setter
+    def src(self, add : InterNetworkAddress | None):
+        if add is not None and not isinstance(add, tuple):
+            raise TypeError(f"Expected tuple or None, got '{type(add).__name__}'")
+        if add is not None and len(add) != 2:
+            raise TypeError(f"Expected a length 2 tuple, got {len(add)}")
+        if add is not None and (not isinstance(add[0], str) or not isinstance(add[1], int)):
+            raise TypeError(f"Expected a (str, int) tuple, got ('{type(add[0]).__name__}', '{type(add[1]).__name__}')")
+        self.__src = add
+
+    @property
+    def dst(self) -> InterNetworkAddress | None:
+        """
+        The address of the destination of the connection as a tuple (ip, port).
+        """
+        return self.__dst
+    
+    @dst.setter
+    def dst(self, add : InterNetworkAddress | None):
+        if add is not None and not isinstance(add, tuple):
+            raise TypeError(f"Expected tuple, got '{type(add).__name__}'")
+        if add is not None and len(add) != 2:
+            raise TypeError(f"Expected a length 2 tuple, got {len(add)}")
+        if add is not None and (not isinstance(add[0], str) or not isinstance(add[1], int)):
+            raise TypeError(f"Expected a (str, int) tuple, got ('{type(add[0]).__name__}', '{type(add[1]).__name__}')")
+        self.__dst = add
 
     @property
     def socket(self) -> "Socket":
@@ -66,20 +111,22 @@ class Connection(UniqueVertex):
 
 
 
-class Socket(UniqueVertex):
+class Socket(DataVertex):
 
     """
     A socket vertex. Represents a handle given by the system to allow some kind of communication.
     """
 
     __slots__ = {
-        "family" : "The socket address family. For example, 'InterNetwork' is for IP V4 addresses. Refer to Socket.families for documentation",
-        "protocol" : "The transport protocol used by the socket. For example, Tcp. Refer to Socket.protocols for documentation",
-        "type" : "The type of socket. For exemple, Dgram is a socket that supports datagrams. Refer to Socket.types for documentation"
+        "__family" : "The socket address family. For example, 'InterNetwork' is for IP V4 addresses. Refer to Socket.families for documentation",
+        "__protocol" : "The transport protocol used by the socket. For example, Tcp. Refer to Socket.protocols for documentation",
+        "__type" : "The type of socket. For exemple, Dgram is a socket that supports datagrams. Refer to Socket.types for documentation",
+        "__handle" : "The handle that represents the socket"
     }
 
-    __pickle_slots__ = {
-        "family",
+    __additional_data__ = DataVertex.__additional_data__ | {
+        "handle",
+        "family", 
         "protocol",
         "type"
     }
@@ -123,6 +170,8 @@ class Socket(UniqueVertex):
         18 : ('VoiceView', 'VoiceView address.'),
     })
 
+    __family_names = {v[0] for v in families.values()}
+
     protocols = defaultdict(lambda : ("Uncharted", "The value does not correspond to any known protocol"), {
         3 : ('Ggp', 'Gateway To Gateway Protocol.'),
         1 : ('Icmp', 'Internet Control Message Protocol.'),
@@ -151,6 +200,8 @@ class Socket(UniqueVertex):
         0 : ('Unspecified', 'Unspecified protocol.'),
     })
 
+    __protocol_names = {v[0] for v in protocols.values()}
+
     types = defaultdict(lambda : ("Uncharted", "The value does not correspond to any known socket type"), {
         2 : ('Dgram', 'Supports datagrams, which are connectionless, unreliable messages of a fixed (typically small) maximum length. Messages might be lost or duplicated and might arrive out of order. A Socket of type Dgram requires no connection prior to sending and receiving data, and can communicate with multiple peers. Dgram uses the Datagram Protocol (ProtocolType.Udp) and the AddressFamily. InterNetwork address family.'),
         3 : ('Raw', 'Supports access to the underlying transport protocol. Using Raw, you can communicate using protocols like Internet Control Message Protocol (ProtocolType.Icmp) and Internet Group Management Protocol (ProtocolType.Igmp). Your application must provide a complete IP header when sending. Received datagrams return with the IP header and options intact.'),
@@ -160,17 +211,78 @@ class Socket(UniqueVertex):
         -1 : ('Unknown', 'Specifies an unknown Socket type.'),
     })
 
-    def __init__(self, *, parent: Optional[Vertex] = None) -> None:
-        super().__init__(parent=parent)
-        self.family : str = ""
-        self.protocol : str = ""
-        self.type : str = ""
+    __type_names = {v[0] for v in types.values()}
+
+    def __init__(self, *, handle : int, family : str = "InterNetwork", protocol : str = "Tcp", type : str = "Stream") -> None:
+        self.__handle = 0
+        self.__family = "Uncharted"
+        self.__protocol = "Uncharted"
+        self.__type = "Uncharted"
+        super().__init__(handle = handle, family = family, protocol = protocol, type = type)
+
+    @property
+    def handle(self) -> int:
+        """
+        The OS handle representing the socket.
+        """
+        return self.__handle
+    
+    @handle.setter
+    def handle(self, h : int):
+        if not isinstance(h, int):
+            raise TypeError(f"Expected int, got '{type(h).__name__}'")
+        self.__handle = h
+    
+    @property
+    def family(self) -> str:
+        """
+        The socket family (example : InterNetwork, Unix, etc.).
+        """
+        return self.__family
+    
+    @family.setter
+    def family(self, f : str):
+        if not isinstance(f, str):
+            raise TypeError(f"Expected str, got '{type(f).__name__}'")
+        if f not in self.__family_names and f != "Uncharted":
+            raise ValueError(f"Invalid family name: '{f}'")
+        self.__family = f
+
+    @property
+    def protocol(self) -> str:
+        """
+        The protocol of the socket (example: IPv4, IPv6, Udp, etc.).
+        """
+        return self.__protocol
+    
+    @protocol.setter
+    def protocol(self, p : str):
+        if not isinstance(p, str):
+            raise TypeError(f"Expected str, got '{type(p).__name__}'")
+        if p not in self.__protocol_names and p != "Uncharted":
+            raise ValueError(f"Invalid family name: '{p}'")
+        self.__protocol = p
+
+    @property
+    def type(self) -> str:
+        """
+        The type of socket (example : Dgram, Stream, ect.).
+        """
+        return self.__type
+    
+    @type.setter
+    def type(self, t : str):
+        if not isinstance(t, str):
+            raise TypeError(f"Expected str, got '{type(t).__name__}'")
+        if t not in self.__type_names and t != "Uncharted":
+            raise ValueError(f"Invalid family name: '{t}'")
+        self.__type = t
 
 
 
 
 
-class Host(UniqueVertex):
+class Host(DataVertex):
 
     """
     A machine vertex. It represents a physical machine.
@@ -194,29 +306,77 @@ class Host(UniqueVertex):
         Host.__current = value
 
     __slots__ = {
-        "address" : "The IP address of the machine",
-        "domain" : "The URL the machine is known as if any",
-        "name" : "The machine's name",
-        "platform" : "The operating system the host is running on"
+        "__address" : "The IP address of the machine",
+        "__domain" : "The URL the machine is known as if any",
+        "__name" : "The machine's name",
+        "__platform" : "The operating system the host is running on"
     }
 
-    __pickle_slots__ = {
+    __defining_data__ = DataVertex.__defining_data__ | {
         "address",
-        "domain",
         "name",
+        "domain",
         "platform"
     }
 
     default_color = ColorSetting(Color.white)
     default_size = SizeSetting(10.0)
 
-    def __init__(self, *, parent: Vertex | None = None) -> None:
-        super().__init__(parent=parent)
-        self.address : str = ""
-        self.domain : str = ""
-        self.name : str = ""
-        self.platform : str = "Unknown"
+    def __init__(self, *, address : str, name : str, domain : str, platform : str) -> None:
+        super().__init__(address = address, name = name, domain = domain, platform = platform)
+
+    @property
+    def address(self) -> str:
+        """
+        The IP address of the Host.
+        """
+        return self.__address
     
+    @address.setter
+    def address(self, a : str):
+        if not isinstance(a, str):
+            raise TypeError(f"Expected str, got '{type(a).__name__}'")
+        self.__address = a
+
+    @property
+    def name(self) -> str:
+        """
+        The Host name.
+        """
+        return self.__name
+
+    @name.setter
+    def name(self, n : str):
+        if not isinstance(n, str):
+            raise TypeError(f"Expected str, got '{type(n).__name__}'")
+        self.__name = n
+
+    @property
+    def domain(self) -> str:
+        """
+        The domain of the Host.
+        """
+        return self.__domain
+    
+    @domain.setter
+    def domain(self, d : str):
+        if not isinstance(d, str):
+            raise TypeError(f"Expected str, got '{type(d).__name__}'")
+        self.__domain = d
+
+    @property
+    def platform(self) -> str:
+        """
+        The Operating System that the Host runs on.
+        """
+        return self.__platform
+    
+    @platform.setter
+    def platform(self, p : str):
+        if not isinstance(p, str):
+            raise TypeError(f"Expected str, got '{type(p).__name__}'")
+        self.__platform = p
+
     @property
     def label(self) -> str:
         """
@@ -230,4 +390,4 @@ class Host(UniqueVertex):
 
 
 
-del Color, ColorSetting, SizeSetting, Optional, UniqueVertex, Vertex, defaultdict, logger, staticproperty
\ No newline at end of file
+del Color, ColorSetting, SizeSetting, Optional, DataVertex, defaultdict, logger, staticproperty
\ No newline at end of file
