@@ -5,7 +5,7 @@ This module contains Edge and Arrow subclasses for this behavioral package.
 from .....logger import logger
 from ...config import SwitchSetting
 from ...colors import Color
-from ...graph import Arrow, Edge, Vertex
+from ...graph import Arrow, DataEdge, Edge, Vertex
 from ..data.entities import Data
 from ..execution.entities import Call, Process
 from .entities import Connection, Host, Socket
@@ -89,17 +89,17 @@ class CreatesSocket(Edge):
 
 
 
-class Binds(Edge):
+class Binds(DataEdge):
 
     """
     This kind of edge indicates that a system call bound a connection to a local address.
     """
 
     __slots__ = {
-        "src" : "The source (local) address of the connection",
+        "__src" : "The source (local) address of the connection",
     }
 
-    __pickle_slots__ = {
+    __defining_data__ = DataEdge.__defining_data__ | {
         "src"
     }
 
@@ -108,27 +108,42 @@ class Binds(Edge):
     source : Call
     destination : Connection
 
-    def __init__(self, source: Vertex, destination: Vertex, *, auto_write: bool = True) -> None:
-        from typing import Any
-        super().__init__(source, destination, auto_write=auto_write)
-        self.src : Any = None
+    def __init__(self, source: Vertex, destination: Vertex, *, auto_write: bool = True, src : tuple[str, int]) -> None:
+        super().__init__(source, destination, auto_write = auto_write, src = src)
+
+    @property
+    def src(self) -> tuple[str, int]:
+        """
+        The local address of the connection.
+        """
+        return self.__src
+    
+    @src.setter
+    def src(self, s : tuple[str, int]):
+        if not isinstance(s, tuple):
+            raise TypeError(f"Expected tuple, got '{type(s).__name__}'")
+        if len(s) != 2:
+            raise ValueError(f"Expected a length 2 tuple, got {len(s)}")
+        if not isinstance(s[0], str) or not isinstance(s[1], int):
+            raise TypeError(f"Expected tuple of str and int, got '{type(s[0]).__name__}' and '{type(s[1]).__name__}'")
+        self.__src = s
 
 
 
 
 
-class Connects(Edge):
+class Connects(DataEdge):
 
     """
     This kind of edge indicates that a system call connected a connection to a remote address.
     """
 
     __slots__ = {
-        "dest" : "The destination (remote) address of the connection"
+        "__dst" : "The destination (remote) address of the connection"
     }
 
-    __pickle_slots__ = {
-        "dest"
+    __defining_data__ = DataEdge.__defining_data__ | {
+        "dst"
     }
 
     label : str = ""
@@ -136,10 +151,25 @@ class Connects(Edge):
     source : Call
     destination : Connection
 
-    def __init__(self, source: Vertex, destination: Vertex, *, auto_write: bool = True) -> None:
-        from typing import Any
-        super().__init__(source, destination, auto_write=auto_write)
-        self.dest : Any = None
+    def __init__(self, source: Vertex, destination: Vertex, *, auto_write: bool = True, dst : tuple[str, int]) -> None:
+        super().__init__(source, destination, auto_write = auto_write, dst = dst)
+
+    @property
+    def dst(self) -> tuple[str, int]:
+        """
+        The remote address of the connection.
+        """
+        return self.__dst
+    
+    @dst.setter
+    def dst(self, d : tuple[str, int]):
+        if not isinstance(d, tuple):
+            raise TypeError(f"Expected tuple, got '{type(d).__name__}'")
+        if len(d) != 2:
+            raise ValueError(f"Expected a length 2 tuple, got {len(d)}")
+        if not isinstance(d[0], str) or not isinstance(d[1], int):
+            raise TypeError(f"Expected tuple of str and int, got '{type(d[0]).__name__}' and '{type(d[1]).__name__}'")
+        self.__dst = d
 
 
 
@@ -244,27 +274,44 @@ class Shutdown(Edge):
 
 
 
-class Accepts(Edge):
+class Accepts(DataEdge):
 
     """
     This kind of arrow indicates that a system call accepted a connection from a remote address.
     """
 
     __slots__ = {
-        "dest" : "The destination (remote) address of the connection"
+        "__dst" : "The destination (remote) address of the connection"
     }
 
-    __pickle_slots__ = {
-        "dest"
+    __defining_data__ = DataEdge.__defining_data__ | {
+        "dst"
     }
 
+    label : str = ""
+
     source : Call
     destination : Connection
 
-    def __init__(self, source: Vertex, destination: Vertex, *, auto_write: bool = True) -> None:
-        from typing import Any
-        super().__init__(source, destination, auto_write=auto_write)
-        self.dest : Any = None
+    def __init__(self, source: Vertex, destination: Vertex, *, auto_write: bool = True, dst : tuple[str, int]) -> None:
+        super().__init__(source, destination, auto_write = auto_write, dst = dst)
+
+    @property
+    def dst(self) -> tuple[str, int]:
+        """
+        The remote address of the connection.
+        """
+        return self.__dst
+    
+    @dst.setter
+    def dst(self, d : tuple[str, int]):
+        if not isinstance(d, tuple):
+            raise TypeError(f"Expected tuple, got '{type(d).__name__}'")
+        if len(d) != 2:
+            raise ValueError(f"Expected a length 2 tuple, got {len(d)}")
+        if not isinstance(d[0], str) or not isinstance(d[1], int):
+            raise TypeError(f"Expected tuple of str and int, got '{type(d[0]).__name__}' and '{type(d[1]).__name__}'")
+        self.__dst = d
 
 
 
@@ -283,4 +330,4 @@ class Duplicates(Arrow):
 
 
 
-del Arrow, Call, Color, SwitchSetting, Connection, Data, Edge, Host, Process, Socket, Vertex, logger
\ No newline at end of file
+del Arrow, DataEdge, Call, Color, SwitchSetting, Connection, Data, Edge, Host, Process, Socket, Vertex, logger
\ No newline at end of file
