@@ -26,13 +26,18 @@ class Key(DataVertex):
     """
 
     __slots__ = {
-        "__name" : "The local name of the registry key int the registry tree."
+        "__name" : "The local name of the registry key int the registry tree.",
+        "__path" : "The full path of the registry key."
     }
 
     __defining_data__ = DataVertex.__defining_data__ | {
         "name"
     }
 
+    __additional_data__ = DataVertex.__additional_data__ | {
+        "path"
+    }
+
     default_color = ColorSetting(Color(0, 150, 255))
     default_size = SizeSetting(1.5)
 
@@ -41,6 +46,7 @@ class Key(DataVertex):
 
     def __init__(self, *, name : str) -> None:
         super().__init__(name = name)
+        self.__path = None
 
     @property
     def name(self) -> str:
@@ -71,9 +77,18 @@ class Key(DataVertex):
         """
         The global key name (path from root key and all sub-keys to reach this key included).
         """
-        if not self.parent_key:
-            return self.name
-        return self.parent_key.path + "\\" + self.name
+        if self.__path is None:
+            if not self.parent_key:
+                self.__path = self.name
+            else:
+                self.__path = self.parent_key.path + "\\" + self.name
+        return self.__path
+    
+    @path.setter
+    def path(self, p : str):
+        if not isinstance(p, str):
+            raise TypeError(f"Expected str, got '{type(p).__name__}'")
+        self.__path = p
     
     def __eq__(self, value: object) -> bool:
         return super().__eq__(value) and self.path == value.path
@@ -312,14 +327,20 @@ class Key_MULTI_SZ_Entry(KeyEntry[tuple[str, ...]]):
             raise TypeError("Expected list of str, got " + repr(type(val).__name__))
         self._set_value(tuple(val))
 
-    def process_value(self, val: str) -> list[str]:
-        if not isinstance(val, str):
-            raise TypeError("Expected str, got " + repr(type(val).__name__))
-        if val.endswith("\x00"):
-            v = val[:-1]
+    def process_value(self, val: str | tuple[str, ...]) -> list[str]:
+        if not isinstance(val, str | tuple):
+            raise TypeError("Expected str or tuple of str, got " + repr(type(val).__name__))
+        if isinstance(val, str):
+            if val.endswith("\x00"):
+                v = val[:-1]
+            else:
+                v = val
+            return v.split("\x00")
         else:
-            v = val
-        return v.split("\x00")
+            for e in val:
+                if not isinstance(e, str):
+                    raise TypeError(f"Expected tuple of str, got a '{type(e).__name__}'")
+            return list(val)
 
 
 
