@@ -5,7 +5,7 @@ All existing MetaGraphs are loaded with the package.
 To create new MetaGraphs, use the module croutons.metalib.interactive.
 """
 
-from ...bakery.logger import logger
+from ...logger import logger
 from .utils import entries, load
 
 logger.info("Loading MetaGraph library.")
@@ -18,7 +18,14 @@ __all__ = []
 
 g = globals()
 for name in entries():
-    g[name] = load(name)
+    try:
+        g[name] = load(name)
+    except:
+        from traceback import print_exc
+        print(f"Error while loading MetaGraph '{name}':")
+        print_exc()
+        del print_exc
+        continue
     __all__.append(name)
     del name
 
