@@ -4,7 +4,8 @@ Look at these classes!
 """
 
 from types import UnionType
-from typing import Any, Iterable, Iterator, Literal, Never, Optional, overload
+from typing import Any, Iterable, Iterator, Literal, Never, Optional, TypeVar, overload
+from Viper.collections.isomorph import IsoDict, IsoSet
 from ...bakery.source.colors import Color
 from ...bakery.source.graph import Arrow, Edge, FrozenGraph, Graph, Vertex, _GraphStateDict
 from .evaluator import Evaluator
@@ -28,6 +29,10 @@ class MetaVertex(Vertex):
     >>> MV = MetaVertex[File]("lambda x : x.name.endswith('.exe')")
     """
 
+    from copy import deepcopy
+    __deepcopy = staticmethod(deepcopy)
+    del deepcopy
+
     @classmethod
     def __class_getitem__(cls, cls_init : type[Vertex] | tuple[type[Vertex], ...] | UnionType):
         try:
@@ -51,7 +56,7 @@ class MetaVertex(Vertex):
         self.__condition : Evaluator[Vertex, bool] | None = None
         self.__color : Color | None = None
         self.__hash : int | None = None
-        self.edges : set[MetaEdge]
+        self.edges : "IsoSet[MetaEdge]"
         super().__init__()
 
     @property
@@ -183,6 +188,20 @@ class MetaVertex(Vertex):
         self.__hash = None
         super().__setstate__(state)
 
+    def __copy__(self):
+        cp = super().__copy__()
+        cp.__hash = None
+        cp.condition = self.condition
+        cp.cls = self.cls
+        return cp
+    
+    def __deepcopy__(self, memo : dict[int, Any]):
+        cp = super().__deepcopy__(memo)
+        cp.__hash = None
+        cp.condition = MetaVertex.__deepcopy(self.condition, memo)
+        cp.cls = MetaVertex.__deepcopy(self.cls, memo)
+        return cp
+
     @property
     def label(self) -> str:
         return f"Vertex[{self.__get_cls_str()}]" + (f"({str(self.__condition)})" if self.__condition else "")
@@ -229,6 +248,10 @@ class MetaEdge(Edge):
     >>> UV = MetaEdge(U, V)[contains]("lambda x : True")
     """
 
+    from copy import deepcopy
+    __deepcopy = staticmethod(deepcopy)
+    del deepcopy
+
     __slots__ = {
         "__class" : "The class that this MetaEdge represents.",
         "__condition" : "An additional condition function. Takes an Edge as an input and tells whether or not it can be a valid match (does not need to check type)",
@@ -372,6 +395,20 @@ class MetaEdge(Edge):
         self.__hash = None
         super().__setstate__(state)
 
+    def __copy__(self):
+        cp = super().__copy__()
+        cp.__hash = None
+        cp.condition = self.condition
+        cp.cls = self.cls
+        return cp
+    
+    def __deepcopy__(self, memo : dict[int, Any]):
+        cp = super().__deepcopy__(memo)
+        cp.__hash = None
+        cp.condition = MetaEdge.__deepcopy(self.condition, memo)
+        cp.cls = MetaEdge.__deepcopy(self.cls, memo)
+        return cp
+
     @property
     def label(self) -> str:
         return f"Edge[{self.__get_cls_str()}]" + (f"({repr(self.__condition.code)})" if self.__condition else "")
@@ -421,6 +458,10 @@ class MetaArrow(Arrow, MetaEdge):
     It has the same syntax rules as MetaEdges.
     """
 
+    from copy import deepcopy
+    __deepcopy = staticmethod(deepcopy)
+    del deepcopy
+
     __slots__ = {
         "__class" : "The class that this MetaArrow represents.",
         "__condition" : "An additional condition function. Takes an Arrow as an input and tells whether or not it can be a valid match (does not need to check type)",
@@ -509,6 +550,20 @@ class MetaArrow(Arrow, MetaEdge):
         self.__hash = None
         super().__setstate__(state)
     
+    def __copy__(self):
+        cp = super().__copy__()
+        cp.__hash = None
+        cp.condition = self.condition
+        cp.cls = self.cls
+        return cp
+    
+    def __deepcopy__(self, memo : dict[int, Any]):
+        cp = super().__deepcopy__(memo)
+        cp.__hash = None
+        cp.condition = MetaArrow.__deepcopy(self.condition, memo)
+        cp.cls = MetaArrow.__deepcopy(self.cls, memo)
+        return cp
+
     @overload
     def match(self, a : Edge) -> Literal[False]:
         ...
@@ -597,7 +652,7 @@ class MetaGraph(Graph):
     }
 
     vertices : __IsoSet[MetaVertex]
-    edges : __IsoSet[MetaEdge | MetaArrow]
+    edges : __IsoSet[MetaEdge]
 
     def __init__(self, g : Iterable[Vertex | Edge] | Graph = Graph()) -> None:
         if isinstance(g, Graph):
@@ -696,6 +751,72 @@ class MetaGraph(Graph):
                     v.write()
                 yield v
         return Graph.extend(self, __checked(), explore)
+    
+    @overload
+    def __or__(self, other: "MetaGraph") -> "MetaGraph":
+        pass
+
+    @overload
+    def __or__(self, other : Graph) -> Graph:
+        pass
+
+    def __or__(self, other):
+        if isinstance(other, MetaGraph):
+            return MetaGraph(Graph.__or__(self, other))
+        return Graph.__or__(self, other)
+    
+    @overload
+    def __ror__(self, other: "MetaGraph") -> "MetaGraph":
+        pass
+
+    @overload
+    def __ror__(self, other : Graph) -> Graph:
+        pass
+
+    def __ror__(self, other):
+        if isinstance(other, MetaGraph):
+            return MetaGraph(Graph.__ror__(other, self))
+        return Graph.__ror__(other, self)
+        
+    @overload
+    def __and__(self, other: "MetaGraph") -> "MetaGraph":
+        pass
+
+    @overload
+    def __and__(self, other : Graph) -> Graph:
+        pass
+
+    def __and__(self, other):
+        if isinstance(other, MetaGraph):
+            return MetaGraph(Graph.__and__(self, other))
+        return Graph.__and__(self, other)
+    
+    @overload
+    def __rand__(self, other: "MetaGraph") -> "MetaGraph":
+        pass
+
+    @overload
+    def __rand__(self, other : Graph) -> Graph:
+        pass
+
+    def __rand__(self, other):
+        if isinstance(other, MetaGraph):
+            return MetaGraph(Graph.__rand__(other, self))
+        return Graph.__rand__(other, self)
+    
+    def __iter__(self) -> Iterator[MetaVertex | MetaEdge]:
+        yield from self.vertices
+        yield from self.edges    
+
+    T = TypeVar("T", bound = MetaVertex | MetaEdge)
+
+    def __getitem__(self, cls : type[T]) -> IsoSet[T]:
+        """
+        Implements self[cls]. Returns an Isoset of vertices or edges of this Graph that are only of the given class(es).
+        """
+        return Graph.__IsoSet(v for v in self.vertices if isinstance(v, cls)) | Graph.__IsoSet(e for e in self.edges if isinstance(e, cls))
+
+    del T
 
     def __dir__(self) -> list[str]:
         return list(super().__dir__()) + self.names
@@ -741,12 +862,14 @@ class MetaGraph(Graph):
                 self.__named_objects.pop(name)
         super().remove(value)
     
-    def __neighborhood_mapper(self, g : Graph) -> __IsoDict[Vertex, __IsoSet[MetaVertex]]:
+    def __neighborhood_mapper(self, g : Graph) -> tuple[__IsoDict[Vertex, __IsoSet[MetaVertex]], __IsoDict[MetaVertex, int]]:
         """
         Searches the given graph for matches of the metagraph.
+        Returns a mapping of the possible roles each Vertex can fit and the number of possible matches for each MetaVertex.
         """
 
         mapping : MetaGraph.__IsoDict[Vertex, MetaGraph.__IsoSet[MetaVertex]] = MetaGraph.__IsoDict()
+        candidates : MetaGraph.__IsoDict[MetaVertex, int] = MetaGraph.__IsoDict((Mv, 0) for Mv in self.vertices)
 
         for v in g.vertices:
             for Mv in self.vertices:
@@ -754,8 +877,9 @@ class MetaGraph(Graph):
                     if v not in mapping:
                         mapping[v] = MetaGraph.__IsoSet()
                     mapping[v].add(Mv)
+                    candidates[Mv] += 1
         
-        return mapping
+        return mapping, candidates
     
     def __clear_mapper_deadends(self, mapping : __IsoDict[Vertex, __IsoSet[MetaVertex]]):
         """
@@ -903,6 +1027,7 @@ class MetaGraph(Graph):
     def search_iter(self, g : Graph) -> Iterator[Graph]:
         """
         Searches through g for all occurences of a subgraph that matches the metagraph.
+        Yields all the matching subgraphs.
         """
         from typing import Iterator
 
@@ -911,7 +1036,9 @@ class MetaGraph(Graph):
         # subgraphs : list[dict[MetaVertex | MetaEdge, Vertex | Edge]] = [{}]
         # next_subgraphs : list[dict[MetaVertex | MetaEdge, Vertex | Edge]] = []
 
-        mapping = self.__neighborhood_mapper(g)
+        mapping, candidates = self.__neighborhood_mapper(g)
+        if 0 in candidates.values():        # At least one MetaVertex has no matches...
+            return
 
         self.__clear_mapper_deadends(mapping)
 
@@ -942,10 +1069,123 @@ class MetaGraph(Graph):
         #     g.extend(dg.values())
         #     yield g
 
+
+
+    class Match:
+
+        """
+        An object that represents a match between a MetaGraph and a Graph.
+        """
+
+        __slots__ = {
+            "__metagraph",
+            "__match",
+            "__map"
+        }
+
+        def __init__(self, metagraph : "MetaGraph", match : Graph, map : IsoDict[MetaVertex | MetaEdge, Vertex | Edge]) -> None:
+            self.__metagraph = metagraph
+            self.__match = match
+            self.__map = map
+
+        @property
+        def metagraph(self) -> "MetaGraph":
+            """
+            The MetaGraph that this is a match for.
+            """
+            return self.__metagraph
+        
+        @property
+        def graph(self) -> Graph:
+            """
+            The complete Graph that matches the MetaGraph.
+            """
+            return self.__match
+        
+        @overload
+        def __getitem__(self, i : MetaVertex) -> Vertex:
+            pass
+
+        @overload
+        def __getitem__(self, i : MetaEdge) -> Edge:
+            pass
+
+        @overload
+        def __getitem__(self, i : MetaArrow) -> Arrow:
+            pass
+
+        @overload
+        def __getitem__(self, i : Vertex) -> MetaVertex:
+            pass
+
+        @overload
+        def __getitem__(self, i : Edge) -> MetaEdge:
+            pass
+
+        def __getitem__(self, i):
+            """
+            Implements self[i].
+            If i is a MetaVertex/MetaEdge, returns the matched Vertex/Edge.
+            If i is a Vertex/Edge, returns the matching MetaVertex/MetaEdge.
+            """
+            if isinstance(i, (MetaVertex, MetaEdge)):
+                if i not in self.__map:
+                    raise KeyError(i)
+                return self.__map[i]
+            else:
+                for Mj, j in self.__map.items():
+                    if i is j:
+                        return Mj
+                raise KeyError(i)
+            
+        def pairs(self):
+            """
+            Iterates over all the match pairs.
+            """
+            return ((i, self[i]) for i in self.__map)
+
+
+
+    def match_iter(self, g : Graph) -> Iterator[Match]:
+        """
+        Searches through g for all occurences of a subgraph that matches the metagraph.
+        Yields all the possible Match objects.
+        """
+        from typing import Iterator
+
+        from ...bakery.source.graph import Graph
+
+        # subgraphs : list[dict[MetaVertex | MetaEdge, Vertex | Edge]] = [{}]
+        # next_subgraphs : list[dict[MetaVertex | MetaEdge, Vertex | Edge]] = []
+
+        mapping, candidates = self.__neighborhood_mapper(g)
+        if 0 in candidates.values():        # At least one MetaVertex has no matches...
+            return
+
+        self.__clear_mapper_deadends(mapping)
+
+        order = list(self.__discover())
+
+        # print("Got {} vertices with openings.".format(len([v for v, sMv in mapping.items() if sMv])))
+
+        def build_iter(g : Graph, mapping : MetaGraph.__IsoDict[Vertex, MetaGraph.__IsoSet[MetaVertex]], sub : MetaGraph.__IsoDict[MetaVertex | MetaEdge, Vertex | Edge], i : int) -> Iterator[tuple[Graph, IsoDict[MetaVertex | MetaEdge, Vertex | Edge]]]:
+            if i == len(order) - 1:     # Last MetaVertex to append
+                for subi in self.__expand_subgraph(g, mapping, sub, order[i]):
+                    gi = Graph()
+                    gi.extend(subi.values())
+                    yield gi, subi
+            else:
+                for subi in self.__expand_subgraph(g, mapping, sub, order[i]):
+                    yield from build_iter(g, mapping, subi, i + 1)
+        
+        yield from (MetaGraph.Match(self, gi, mapi) for gi, mapi in build_iter(g, mapping, MetaGraph.__IsoDict(), 0))
+
     def __contains__(self, g : Graph) -> bool:
         """
         Implements g in self. Returns True if Graph g matches entirely MetaGraph self.
         """
+        if len(g.vertices) > len(self.vertices) or len(g.edges) > len(self.edges):
+            return False
         return g in self.search_iter(g)
 
 
@@ -963,8 +1203,8 @@ class FrozenMetaGraph(MetaGraph, FrozenGraph):
 
     def __init__(self, g: Iterable[Vertex | Edge] | Graph = Graph()) -> None:
         MetaGraph.__init__(self, g)
-        self.vertices : "FrozenMetaGraph.__FrozenIsoSet[Vertex]" = FrozenMetaGraph.__FrozenIsoSet(self.vertices)
-        self.edges : "FrozenMetaGraph.__FrozenIsoSet[Edge]" = FrozenMetaGraph.__FrozenIsoSet(self.edges)
+        self.vertices : "FrozenMetaGraph.__FrozenIsoSet[MetaVertex]" = FrozenMetaGraph.__FrozenIsoSet(self.vertices)
+        self.edges : "FrozenMetaGraph.__FrozenIsoSet[MetaEdge]" = FrozenMetaGraph.__FrozenIsoSet(self.edges)
 
     def append(self, value: MetaVertex | MetaEdge, explore: bool = False) -> Never:
         raise AttributeError(f"Cannot append to a '{type(self).__name__}'")
@@ -975,6 +1215,106 @@ class FrozenMetaGraph(MetaGraph, FrozenGraph):
     def extend(self, values: Iterable[MetaVertex | MetaEdge], explore: bool = False) -> Never:
         raise AttributeError(f"Cannot extend a '{type(self).__name__}'")
     
+    @overload
+    def __or__(self, other: "FrozenMetaGraph") -> "FrozenMetaGraph":
+        pass
+
+    @overload
+    def __or__(self, other : MetaGraph) -> MetaGraph:
+        pass
+
+    @overload
+    def __or__(self, other : FrozenGraph) -> FrozenGraph:
+        pass
+
+    @overload
+    def __or__(self, other : Graph) -> Graph:
+        pass
+
+    def __or__(self, other):
+        if isinstance(other, FrozenMetaGraph):
+            return FrozenMetaGraph(Graph.__or__(self, other))
+        elif isinstance(other, MetaGraph):
+            return MetaGraph(Graph.__or__(self, other))
+        elif isinstance(other, FrozenMetaGraph.__FrozenGraph):
+            return FrozenMetaGraph.__FrozenGraph(Graph.__or__(self, other))
+        return Graph.__or__(self, other)
+
+    @overload
+    def __ror__(self, other: "FrozenMetaGraph") -> "FrozenMetaGraph":
+        pass
+
+    @overload
+    def __ror__(self, other : MetaGraph) -> MetaGraph:
+        pass
+
+    @overload
+    def __ror__(self, other : FrozenGraph) -> FrozenGraph:
+        pass
+
+    @overload
+    def __ror__(self, other : Graph) -> Graph:
+        pass
+
+    def __ror__(self, other):
+        if isinstance(other, FrozenMetaGraph):
+            return FrozenMetaGraph(Graph.__ror__(self, other))
+        elif isinstance(other, MetaGraph):
+            return MetaGraph(Graph.__ror__(self, other))
+        elif isinstance(other, FrozenMetaGraph.__FrozenGraph):
+            return FrozenMetaGraph.__FrozenGraph(Graph.__ror__(self, other))
+        return Graph.__ror__(self, other)
+    
+    @overload
+    def __and__(self, other: "FrozenMetaGraph") -> "FrozenMetaGraph":
+        pass
+
+    @overload
+    def __and__(self, other : MetaGraph) -> MetaGraph:
+        pass
+
+    @overload
+    def __and__(self, other : FrozenGraph) -> FrozenGraph:
+        pass
+
+    @overload
+    def __and__(self, other : Graph) -> Graph:
+        pass
+
+    def __and__(self, other):
+        if isinstance(other, FrozenMetaGraph):
+            return FrozenMetaGraph(Graph.__and__(self, other))
+        elif isinstance(other, MetaGraph):
+            return MetaGraph(Graph.__and__(self, other))
+        elif isinstance(other, FrozenMetaGraph.__FrozenGraph):
+            return FrozenMetaGraph.__FrozenGraph(Graph.__and__(self, other))
+        return Graph.__and__(self, other)
+
+    @overload
+    def __rand__(self, other: "FrozenMetaGraph") -> "FrozenMetaGraph":
+        pass
+
+    @overload
+    def __rand__(self, other : MetaGraph) -> MetaGraph:
+        pass
+
+    @overload
+    def __rand__(self, other : FrozenGraph) -> FrozenGraph:
+        pass
+
+    @overload
+    def __rand__(self, other : Graph) -> Graph:
+        pass
+
+    def __rand__(self, other):
+        if isinstance(other, FrozenMetaGraph):
+            return FrozenMetaGraph(Graph.__rand__(self, other))
+        elif isinstance(other, MetaGraph):
+            return MetaGraph(Graph.__rand__(self, other))
+        elif isinstance(other, FrozenMetaGraph.__FrozenGraph):
+            return FrozenMetaGraph.__FrozenGraph(Graph.__rand__(self, other))
+        return Graph.__rand__(self, other)
+
     def __getstate__(self) -> _MetaGraphStateDict:
         return MetaGraph.__getstate__(self) | FrozenMetaGraph.__FrozenGraph.__getstate__(self)
     
